;------------------------------------------------------------------------
;                                                                       |
;   FILE        :ir_rcv.asm                                             |
;   DATE        :Wed, Nov 26, 2008                                      |
;   DESCRIPTION :Main Program                                           |
;   CPU TYPE    :PIC12F683                                              |
;                                                                       |
;   This file is programed by TOYO-LINX Co.,Ltd. / yKikuchi             |
;                                                                       |
;------------------------------------------------------------------------

	list      p=12f683
	#include <p12f683.inc>

	__CONFIG	_FCMEN_ON & _IESO_ON & _BOD_ON & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT

	RADIX	DEC

;************************************************************************
;	Constant Diffinition
;************************************************************************
IR		equ		GP2		;IR Signal Bit
TIMING	equ		GP0		;Timig Check Signal

;************************************************************************
;	Data Memory
;************************************************************************
	UDATA	20h
ircmd		RES		1	;IR Command
ircode		RES		1	;IR Signal Code
ircnt		RES		1	;IR Receive Counter

txdata		RES		1	;Transimt Data
txcnt		RES		1	;Transmit Counter
tx_tmp		RES		1	;Test Program Data

wait_tmp	RES		1	;Wait Counter
wait_tmp2	RES		1	;Wait Counter No.2

;************************************************************************
;	Program Memory
;************************************************************************
	ORG	0000h
	GOTO	start

	ORG	0004h

;************************************************************************
;	Initialize
;************************************************************************
	ORG	0010h
start
	BSF		STATUS,RP0	;Bank=1
	MOVLW	01111000b	;Internal OSC 8MHz
	MOVWF	OSCCON
	BCF		STATUS,RP0	;Bank=0
	MOVLW	000010b		;GPIO Initial Out
	MOVWF	GPIO
	MOVLW	07h			;Comparator Off(Use I/O Pin)
	MOVWF	CMCON0
	BSF		STATUS,RP0	;Bank=1
	CLRF	ANSEL		;Analog Input Off(Use I/O Pin)
	MOVLW	111100b		;GP2,3,4,5=Input / GP0,1=Output
	MOVWF	TRISIO
	MOVLW	01111111b	;GPIO Pull Up On
	MOVWF	OPTION_REG
	BCF		STATUS,RP0	;Bank=0

;************************************************************************
;	Main Program
;************************************************************************
main
	CLRWDT

	BTFSC	GPIO,IR		;Leader Code
	GOTO	main
	CALL	check_leader
	SUBLW	01h
	BTFSS	STATUS,Z
	GOTO	main

	CALL	getcode		;Custom Code(1)
	MOVLW	000h
	SUBWF	ircode,W
	BTFSS	STATUS,Z
	GOTO	main

	CALL	getcode		;Custom Code(2)
	MOVLW	0ffh
	SUBWF	ircode,W
	BTFSS	STATUS,Z
	GOTO	main

	CALL	getcode		;Data
	MOVF	ircode,W
	MOVWF	ircmd

	CALL	getcode		;Data(Compliment)
	COMF	ircode,W
	SUBWF	ircmd,W
	BTFSS	STATUS,Z
	GOTO	main

	MOVF	ircmd,W		;Transmit Command Data
	MOVWF	txdata
	CALL	txone

	GOTO	main

;************************************************************************
;	Check IR Leader Signal
;************************************************************************
check_leader
	MOVLW	250
	MOVWF	wait_tmp
check_leader_01
	BTFSC	GPIO,IR
	GOTO	check_leader_40
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	DECFSZ	wait_tmp,F
	GOTO	check_leader_01
		
	CLRF	wait_tmp
check_leader_11
	BTFSC	GPIO,IR
	GOTO	check_leader_21
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	DECFSZ	wait_tmp,F
	GOTO	check_leader_11
	GOTO	check_leader_40

check_leader_21
	MOVLW	250
	MOVWF	wait_tmp
check_leader_22
	BTFSS	GPIO,IR
	GOTO	check_leader_40
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	DECFSZ	wait_tmp,F
	GOTO	check_leader_22

	CLRF	wait_tmp
check_leader_31
	BTFSS	GPIO,IR
	RETLW	01h			;IR Leader Signal Get
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	DECFSZ	wait_tmp,F
	GOTO	check_leader_31
	GOTO	check_leader_40

check_leader_40
	RETLW	00h			;IR Leader Signal NG

;************************************************************************
;	Get IR Signal Code
;************************************************************************
getcode
	MOVLW	8
	MOVWF	ircnt
	CLRF	ircode
		
getcode_01
	BTFSS	GPIO,IR
	GOTO	$-1

	BSF		GPIO,TIMING
	CALL	wait15bit
	BCF		GPIO,TIMING
		
	BTFSC	GPIO,IR
	GOTO	getcode_03
	BCF		STATUS,C
	RRF		ircode,F
getcode_02
	DECFSZ	ircnt,F	; 12
	GOTO	getcode_01
	RETURN
getcode_03
	BSF		STATUS,C
	RRF		ircode,F
		
	BTFSC	GPIO,IR
	GOTO	$-1
	GOTO	getcode_02

;************************************************************************
;	Wait 1.5bit(IR Signal)
;************************************************************************
wait15bit
	MOVLW	209			;IR Signal 1.5bit(0.84ms)
	MOVWF	wait_tmp
wait15bit_01
	CLRWDT
	GOTO	$+1
	GOTO	$+1
	DECFSZ	wait_tmp,F
	GOTO	wait15bit_01
	RETURN

;************************************************************************
;	Transmit 1-Charactor
;************************************************************************
TXD		equ		GP1		;TXD Bit

txone
	MOVLW	8
	MOVWF	txcnt

	BCF		GPIO,TXD	;Start Bit
	CALL	wait1bit

txone_01
	RRF		txdata,F	;Data Bit
	BTFSC	STATUS,C
	GOTO	txone_02
	BCF		GPIO,TXD
	GOTO	txone_03
txone_02
	BSF		GPIO,TXD
	NOP
txone_03
	CALL	wait1bit
	DECFSZ	txcnt,F
	GOTO	txone_01

	BSF		GPIO,TXD	;Stop Bit(2bit)
	CALL	wait1bit
	CALL	wait1bit

	RETURN

;************************************************************************
;	Wait 1-bit
;************************************************************************
wait1bit
	MOVLW	10			;38400 baud
	MOVWF	wait_tmp
wait1bit_01
	CLRWDT
	DECFSZ	wait_tmp,F
	GOTO	wait1bit_01
	RETURN

;************************************************************************
;	Wait Timer
;************************************************************************
wait1ms
	MOVLW	249			;1ms
	MOVWF	wait_tmp
wait1ms_01
	CLRWDT
	GOTO	$+1
	GOTO	$+1
	DECFSZ	wait_tmp,F
	GOTO	wait1ms_01
	RETURN
	
wait100ms
	MOVLW	100			;100ms
wait100ms_00
	MOVWF	wait_tmp2
wait100ms_01
	CALL	wait1ms
	DECFSZ	wait_tmp2,F
	GOTO	wait100ms_01
	RETURN

wait200ms
	MOVLW	200			;200ms
	GOTO	wait100ms_00

;************************************************************************
;	Test Program 1
;************************************************************************
test01
	BSF		GPIO,1
	BCF		GPIO,1
	GOTO	test01

;************************************************************************
;	Test Program 2
;************************************************************************
test02
	MOVLW	020h
	MOVWF	tx_tmp
test02_01
	MOVF	tx_tmp,W
	MOVWF	txdata
	call	txone
	INCF	tx_tmp,F
	BTFSS	tx_tmp,6
	GOTO	test02_01
	MOVLW	0dh
	MOVWF	txdata
	CALL	txone
test02_02
	MOVF	tx_tmp,W
	MOVWF	txdata
	call	txone
	INCF	tx_tmp,F
	BTFSS	tx_tmp,7
	GOTO	test02_02
	MOVLW	0dh
	MOVWF	txdata
	CALL	txone
	CALL	wait200ms
	CALL	wait200ms
	CALL	wait100ms
	GOTO	test02

;************************************************************************
	END
